@@ -34,7 +34,7 @@
 #include "iopause.h"
 
 #define USAGE " [-ttv] [-r c] [-R abc] [-l len] [-b buflen] dir ..."
-#define VERSION "$Id: svlogd.c,v 1.17 2005/12/18 11:15:51 pape Exp $"
+#define VERSION "$Id: svlogd.c,v 1.19 2006/06/19 19:55:08 pape Exp $"
 
 #define FATAL "svlogd: fatal: "
 #define WARNING "svlogd: warning: "
@@ -93,6 +93,7 @@ struct logdir {
   char matcherr;
   struct sockaddr_in udpaddr;
   unsigned int udponly;
+  stralloc prefix;
 } *dir;
 unsigned int dirn =0;
 
@@ -199,8 +200,6 @@ unsigned int processorstop(struct logdir *ld) {
       pause1("unable to change to initial working directory");
     return(ld->processor.len ? 0 : 1);
   }
-  if (unlink(ld->fnsave) == -1)
-    strerr_warn5(WARNING, "unable to unlink: ", ld->name, "/", ld->fnsave, 0);
   ld->fnsave[26] ='t';
   byte_copy(f, 26, ld->fnsave);
   f[26] ='s'; f[27] =0;
@@ -208,6 +207,9 @@ unsigned int processorstop(struct logdir *ld) {
     pause2("unable to rename processed", ld->name);
   while (chmod(f, 0744) == -1)
     pause2("unable to set mode of processed", ld->name);
+  ld->fnsave[26] ='u';
+  if (unlink(ld->fnsave) == -1)
+    strerr_warn5(WARNING, "unable to unlink: ", ld->name, "/", ld->fnsave, 0);
   while (rename("newstate", "state") == -1)
     pause2("unable to rename state", ld->name);
   if (verbose) strerr_warn5(INFO, "processed: ", ld->name, "/", f, 0);
@@ -216,12 +218,40 @@ unsigned int processorstop(struct logdir *ld) {
   return(1);
 }
 
-unsigned int rotate(struct logdir *ld) {
+void rmoldest(struct logdir *ld) {
   DIR *d;
   direntry *f;
-  int n =0;
-  char tmp[FMT_ULONG +1];
   char oldest[FMT_PTIME];
+  int n =0;
+
+  oldest[0] ='A'; oldest[1] =oldest[27] =0;
+  while (! (d =opendir(".")))
+    pause2("unable to open directory, want rotate", ld->name);
+  errno =0;
+  while ((f =readdir(d)))
+    if ((f->d_name[0] == '@') && (str_len(f->d_name) == 27)) {
+      if (f->d_name[26] == 't') {
+        if (unlink(f->d_name) == -1)
+          warn2("unable to unlink processor leftover", f->d_name);
+      }
+      else {
+        ++n;
+        if (str_diff(f->d_name, oldest) < 0) byte_copy(oldest, 27, f->d_name);
+      }
+      errno =0;
+    }
+  if (errno) warn2("unable to read directory", ld->name);
+  closedir(d);
+
+  if (ld->nmax && (n > ld->nmax)) {
+    if (verbose) strerr_warn5(INFO, "delete: ", ld->name, "/", oldest, 0);
+    if ((*oldest == '@') && (unlink(oldest) == -1))
+      warn2("unable to unlink oldest logfile", ld->name);
+  }
+}
+
+unsigned int rotate(struct logdir *ld) {
+  char tmp[FMT_ULONG +1];
 
   if (ld->fddir == -1) { ld->tmax =0; return(0); }
   if (ld->ppid) while(! processorstop(ld));
@@ -268,24 +298,7 @@ unsigned int rotate(struct logdir *ld) {
     ld->size =0;
     while (fchmod(ld->fdcur, 0644) == -1)
       pause2("unable to set mode of current", ld->name);
-    
-    oldest[0] ='A'; oldest[1] =oldest[27] =0;
-    while (! (d =opendir(".")))
-      pause2("unable to open directory, want rotate", ld->name);
-    errno =0;
-    while ((f =readdir(d)))
-      if ((f->d_name[0] == '@') && (str_len(f->d_name) == 27)) {
-        ++n;
-        if (str_diff(f->d_name, oldest) < 0) byte_copy(oldest, 27, f->d_name);
-      }
-    if (errno) warn2("unable to read directory", ld->name);
-    closedir(d);
-    
-    if (ld->nmax && (n >= ld->nmax)) {
-      if (verbose) strerr_warn5(INFO, "delete: ", ld->name, "/", oldest, 0);
-      if ((*oldest == '@') && (unlink(oldest) == -1))
-        warn2("unable to unlink oldest logfile", ld->name);
-    }
+    rmoldest(ld);
     processorstart(ld);
   }
 
@@ -419,6 +432,7 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
   ld->match ='+';
   ld->udpaddr.sin_port =0;
   ld->udponly =0;
+  while (! stralloc_copys(&ld->prefix, "")) pause_nomem();
   while (! stralloc_copys(&ld->inst, "")) pause_nomem();
   while (! stralloc_copys(&ld->processor, "")) pause_nomem();
 
@@ -431,9 +445,7 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
 
     if (verbose) strerr_warn4(INFO, "read: ", ld->name, "/config", 0);
     for (i =0; i +1 < sa.len; ++i) {
-      if ((len =byte_chr(&sa.s[i], sa.len -i, '\n')) == 1) {
-        ++i; continue;
-      }
+      len =byte_chr(&sa.s[i], sa.len -i, '\n');
       sa.s[len +i] =0;
       switch(sa.s[i]) {
       case '\n':
@@ -473,8 +485,10 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
         }
         break;
       case '!':
-        while (! stralloc_copys(&ld->processor, &sa.s[i +1])) pause_nomem();
-        while (! stralloc_0(&ld->processor)) pause_nomem();
+        if (len > 1) {
+          while (! stralloc_copys(&ld->processor, &sa.s[i +1])) pause_nomem();
+          while (! stralloc_0(&ld->processor)) pause_nomem();
+        }
         break;
       case 'U':
         ld->udponly =1;
@@ -494,7 +508,7 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
           port =514;
         ld->udpaddr.sin_port =htons(port);
         if (fdudp == -1) {
-                 fdudp =socket(AF_INET, SOCK_DGRAM, 0);
+          fdudp =socket(AF_INET, SOCK_DGRAM, 0);
           if (fdudp)
             if (ndelay_on(fdudp) == -1) {
               close(fdudp);
@@ -502,6 +516,12 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
             }
         }
         break;
+      case 'p':
+        if (len > 1) {
+          while (! stralloc_copys(&ld->prefix, &sa.s[i +1])) pause_nomem();
+          while (! stralloc_0(&ld->prefix)) pause_nomem();
+        }
+        break;
       }
       i +=len;
     }
@@ -518,6 +538,7 @@ unsigned int logdir_open(struct logdir *ld, const char *fn) {
       } while ((stat(ld->fnsave, &st) != -1) || (errno != error_noent));
       while (rename("current", ld->fnsave) == -1)
         pause2("unable to rename current", ld->name);
+      rmoldest(ld);
       i =-1;
     }
     else
@@ -723,7 +744,6 @@ int main(int argc, const char **argv) {
     char ch;
 
     linelen =0;
-    if (exitasap && ! data.p) break; /* data buffer is empty */
     for (linelen =0; linelen < linemax; ++linelen) {
       if (buffer_GETC(&data, &ch) <= 0) {
         exitasap =1;
@@ -755,12 +775,13 @@ int main(int argc, const char **argv) {
       }
       line[linelen] =ch;
     }
-    if (! linelen) continue;
+    if (exitasap && ! data.p) break; /* data buffer is empty */
     for (i =0; i < dirn; ++i)
       if (dir[i].fddir != -1) {
         if (dir[i].inst.len) logmatch(&dir[i]);
         if (dir[i].matcherr == 'e') {
           if (timestamp) buffer_puts(buffer_2, stamp);
+          if (dir[i].prefix.len) buffer_puts(buffer_2, dir[i].prefix.s);
           buffer_put(buffer_2, line, linelen);
           if (linelen == linemax) buffer_puts(buffer_2, "...");
           buffer_put(buffer_2, "\n", 1); buffer_flush(buffer_2);
@@ -769,28 +790,34 @@ int main(int argc, const char **argv) {
         if (dir[i].udpaddr.sin_port != 0) {
           if (fdudp == -1) {
             buffer_puts(&dir[i].b, "warning: no udp socket available: ");
+            if (timestamp) buffer_puts(&dir[i].b, stamp);
+            if (dir[i].prefix.len) buffer_puts(&dir[i].b, dir[i].prefix.s);
             buffer_put(&dir[i].b, line, linelen);
             buffer_put(&dir[i].b, "\n", 1);
             buffer_flush(&dir[i].b);
           }
           else {
-            if (linelen >= linemax -1) {
-              line[linemax -4] =line[linemax -3] =line[linemax -2] ='.';
-              linelen =linemax -1;
-            }
-            if (line[linelen -1] != '\n') line[linelen++] ='\n';
-            if (sendto(fdudp, line, linelen, 0,
+            while (! stralloc_copys(&sa, "")) pause_nomem();
+            if (timestamp)
+              while (! stralloc_cats(&sa, stamp)) pause_nomem();
+            if (dir[i].prefix.len)
+              while (! stralloc_cats(&sa, dir[i].prefix.s)) pause_nomem();
+            while (! stralloc_catb(&sa, line, linelen)) pause_nomem();
+            if (linelen == linemax)
+              while (! stralloc_cats(&sa, "...")) pause_nomem();
+            while (! stralloc_append(&sa, "\n")) pause_nomem();
+            if (sendto(fdudp, sa.s, sa.len, 0,
                        (struct sockaddr *)&dir[i].udpaddr,
-                       sizeof(dir[i].udpaddr)) != linelen) {
+                       sizeof(dir[i].udpaddr)) != sa.len) {
               buffer_puts(&dir[i].b, "warning: failure sending through udp: ");
-              buffer_put(&dir[i].b, line, linelen);
+              buffer_put(&dir[i].b, sa.s, sa.len);
               buffer_flush(&dir[i].b);
             }
-	    if (line[linelen -1] == '\n') --linelen;
           }
         }
         if (! dir[i].udponly) {
           if (timestamp) buffer_puts(&dir[i].b, stamp);
+          if (dir[i].prefix.len) buffer_puts(&dir[i].b, dir[i].prefix.s);
           buffer_put(&dir[i].b, line, linelen);
         }
       }
